@@ -19,7 +19,7 @@ use Net::Proxmox::VE::Cluster;
 use Net::Proxmox::VE::Nodes;
 
 
-our $VERSION = 0.006;
+our $VERSION = 0.009;
 
 =encoding utf8
 
@@ -34,14 +34,14 @@ Net::Proxmox::VE - Pure perl API for Proxmox virtualisation
     %args = (
         host     => 'proxmox.local.domain',
         password => 'barpassword',
-        user     => 'root', # optional
+        username => 'root', # optional
         port     => 8006,   # optional
         realm    => 'pam',  # optional
     );
 
     $host = Net::Proxmox::VE->new(%args);
 
-    $host->login() or die ('Couldnt log in to proxmox host');
+    $host->login() or die ('Couldn\'t log in to proxmox host');
 
 =head1 WARNING
 
@@ -77,6 +77,11 @@ This class provides the building blocks for someone wanting to use PHP to talk t
 
 =head1 METHODS
 
+=head2 action
+
+This calls raw actions against your proxmox server.
+Ideally you don't use this directly.
+
 =cut
 
 sub action {
@@ -157,12 +162,6 @@ sub action {
         my $content = $response->decoded_content;
         my $data    = decode_json( $response->decoded_content );
 
-        # DELETE operations return no data
-        # otherwise if we have a data key but its empty, treat it as a failure
-        if ( $params{method} eq 'DELETE' ) {
-            return 1;
-        }
-
         if ( ref $data eq 'HASH'
             && exists $data->{data} )
         {
@@ -281,9 +280,11 @@ sub get {
         if ref $_[-1];
     my @path = @_    or return;    # using || breaks this
 
-    if ( $self->nodes ) {
+    # Calling nodes method here would call get method itself and so on
+    # Commented out to avoid an infinite loop
+    #if ( $self->nodes ) {
         return $self->action( path => join( '/', @path ), method => 'GET', post_data => $post_data );
-    }
+    #}
     return;
 }
 
@@ -320,6 +321,28 @@ TCP port number used to by the Proxmox host instance. Defaults to 8006, optional
 
 Authentication realm to request against. Defaults to 'pam' (local auth), optional.
 
+=item I<ssl_opts>
+
+If you're using a self-signed certificate, SSL verification is going to fail, and we need to tell C<IO::Socket::SSL> not to attempt certificate verification.
+
+This option is passed on as C<ssl_opts> options to C<LWP::UserAgent-E<gt>new()>, ultimately for C<IO::Socket::SSL>.
+
+Using it like this, causes C<LWP::UserAgent> and C<IO::Socket::SSL> not to attempt SSL verification:
+
+    use IO::Socket::SSL qw(SSL_VERIFY_NONE);
+    ..
+    %args = (
+        ...
+        ssl_opts => {
+            SSL_verify_mode => SSL_VERIFY_NONE,
+            verify_hostname => 0
+        },
+        ...
+    );
+    my $proxmox = Net::Proxmox::VE->new(%args);
+
+Your connection will work now, but B<beware: you are now susceptible to a man-in-the-middle attack>.
+
 =item I<debug>
 
 Enabling debugging of this API (not related to proxmox debugging in any way). Defaults to false, optional.
@@ -360,6 +383,7 @@ sub new {
     $params{username} ||= 'root';
     $params{realm}    ||= 'pam';
     $params{debug}    ||= undef;
+    $params{timeout}  ||= 10;
 
     my $self->{params} = \%params;
     $self->{'ticket'}           = undef;
@@ -467,14 +491,18 @@ http://pve.proxmox.com/pve2-api-doc
 
  Contribute at http://github.com/mrproper/proxmox-ve-api-perl
 
+=head1 LICENSE
+ This program is free software; you can redistribute it and/or modify it under
+ the same terms as Perl itself.
+
 =head1 VERSION
 
- 0.001
+ 0.009
 
 =head1 AUTHORS
 
  Brendan Beveridge <brendan@nodeintegration.com.au>
- Dean Hamstead <dean@fragfest.com.au>
+ Dean Hamstead <dean@bytefoundry.com.au>
 
 =cut
 
