@@ -859,7 +859,15 @@ sub login {
     # Prepare login request
     my $url = $self->url_prefix . '/api2/json/access/ticket';
 
-    my $ua = LWP::UserAgent->new( ssl_opts => { verify_hostname => undef } );
+    my %lwpUserAgentOptions;
+    if ($self->{params}->{ssl_opts}) {
+        $lwpUserAgentOptions{ssl_opts} = $self->{params}->{ssl_opts};
+    }
+
+    my $ua = LWP::UserAgent->new( %lwpUserAgentOptions );
+
+    $ua->timeout($self->{params}->{timeout});
+
     $self->{ua} = $ua;
 
     # Perform login request
@@ -890,6 +898,8 @@ sub login {
 
         print "DEBUG: login not successful\n"
           if $self->{params}->{debug};
+        print "DEBUG: " . $response->status_line . "\n"
+          if $self->{params}->{debug};
 
     }
 
@@ -1019,7 +1029,7 @@ sub update_access_password {
 =head1 AUTHORS
 
   Brendan Beveridge L<<brendan@nodeintegration.com.au>>
-  Dean Hamstead L<<dean@fragfest.com.au>>
+  Dean Hamstead L<<dean@bytefoundry.com.au>>
 
 =cut
 
