@@ -19,7 +19,6 @@ Net::Proxmox::VE::Cluster - Functions for the 'cluster' portion of the API
 
   # assuming $obj is a Net::Proxmox::VE object
 
-
 =head1 METHODS
 
 =head2 cluster
@@ -839,7 +838,7 @@ sub get_cluster_tasks {
 
 =head1 AUTHORS
 
-  Dean Hamstead L<<dean@fragfest.com.au>>
+  Dean Hamstead L<<dean@bytefoundry.com.au>>
 
 =cut
 
