@@ -219,7 +219,7 @@ sub update_pool {
 
 =head1 AUTHOR
 
- Dean Hamstead L<<dean@fragfest.com.au>>
+ Dean Hamstead L<<dean@bytefoundry.com.au>>
 
 =cut
 
